def keitaro_parser(api_key, host, firstdate='', lastdate='', interval = '30_days_ago', timezone = 'Europe/Moscow', filters = [], grouping = ['stream', 'offer_group'], metrics = ['sub_id_12','clicks', 'campaign_unique_clicks', 'sale_revenue', 'epc_confirmed', 'approve', 'sales']):
    
    """
    Функция позволяет парсить данные с кейтаро по запросу. Порядок действий следующий, надо формировать отчёт в кейтаро, затем забирать необходимые данные как указано в гайде: https://docs.keitaro.io/ru/development/admin-api.html
    
    api_key: (str) - апи-ключ для доступа в кейтаро
    host: (str) - url для доступа к кейтаро
    firstdate: (str) - дата, с которой необходимо начать отчёт формта YYYY-MM-DD, либо пустота, если interval не равен custom_date_range
    lastade: (str) - дата, на которой необходимо закончить отчёт формта YYYY-MM-DD, либо пустота, если interval не равен custom_date_range
    interval: (str) - за какой период взяты данные (берётся из кейтаро)
    timezone: (str) - Параметр который не надо менять
    filters: list(dictionary()) - список из словарей, копировать строго из кейтаро
    grouping: list() - параметр, по которому осуществляется группировка (брать строго из кейтаро)
    metrics: list() - метрики, выбираемые в настройках кейтаро 
    
    """
    
    import requests
    import json
    import pandas as pd
    import numpy as np
    import matplotlib.pyplot as plt
    api_url = f'{host}admin_api/v1/'
    if (interval != 'custom_date_range') & ((interval != 'custom_time_range')):
        params =  {
            'range': {
            'interval': interval,
            },
            'timezone': timezone,
            'filters': []+filters,
            'grouping': grouping,
            'metrics': metrics
            }

    else:
        params = {
            'range': {
                'from': firstdate,
                'interval': interval,
                'timezone': timezone,
                'to': lastdate
            },
            'filters': []+filters,
            'grouping': grouping,
            'metrics': metrics
            }
        
    headers = {'Api-Key': api_key}

    response = requests.post(
        url=f'{api_url}report/build',
        headers=headers,
        json=params
    )

    parsed_response = json.loads(response.text)

    df_per_days = pd.DataFrame(parsed_response['rows'])
    if 'day' in grouping:
        df_per_days['day'] = pd.to_datetime(df_per_days['day'])
    elif 'datetime' in grouping:
        df_per_days['datetime'] = pd.to_datetime(pd.to_datetime(df_per_days['datetime']).dt.date)
    elif 'day_hour' in grouping:
        df_per_days['day_hour'] = pd.to_datetime(pd.to_datetime(df_per_days['day_hour'], format = "%Y%m%d%H"))
    if 'sub_id' in df_per_days.columns:
        df_per_days = df_per_days.drop('sub_id', axis = 1)
    return(df_per_days)                


def plot_acf_pacf(data,lags=30):    
    """
    Строит графики ACF и PACF для временного ряда.

    Args:
        data: Временной ряд (pandas Series или numpy array).
        lags: Максимальное количество лагов для построения графиков (по умолчанию 30).
    """
    import matplotlib.pyplot as plt
    import numpy as np
    from statsmodels.tsa.stattools import acf, pacf
    
    # Вычисляем ACF и PACF
    acf_values = acf(data, nlags=lags)
    pacf_values = pacf(data, nlags=lags)

    # Строим графики
    fig, axs = plt.subplots(2, 1, figsize=(10, 5))

    axs[0].plot(acf_values, marker = 'o')
    axs[0].axhline(y=0, linestyle='--', color='gray')
    axs[0].axhline(y=-1.96/np.sqrt(len(data)), linestyle='--', color='gray')
    axs[0].axhline(y=1.96/np.sqrt(len(data)), linestyle='--', color='gray')
    axs[0].set_title('ACF')
    axs[0].set_xlabel('Lag')

    axs[1].plot(pacf_values, marker = 'o')
    axs[1].axhline(y=0, linestyle='--', color='gray')
    axs[1].axhline(y=-1.96/np.sqrt(len(data)), linestyle='--', color='gray')
    axs[1].axhline(y=1.96/np.sqrt(len(data)), linestyle='--', color='gray')
    axs[1].set_title('PACF')
    axs[1].set_xlabel('Lag')

    plt.tight_layout()
    plt.show()
    
    
def plot_external(params_chan, params_all, type_, external_name):
    """
    Строит графики для сравнения динамики по каким-то каналам (витринам\офферам\потомкам)

    Args:
        sub_14: list() - список из необходимых sub_14 (нужен и sms и feed).
        type_: string - FS, если необходимо сравнить feed и sms в рамках external: FF, если необходимо сравнить feed всего трафика и feed в рамках external; SS если необходимо сравнить smsвсего трафика и sms в рамках external.
        external_name: string - Название external
        params_chan: dict() - параметры запроса в keitro_parser в external для считывания данных по уникальному каналу, который необходимо посмотреть (витрина\оффер\поток и тд) (в metrics обязана быть epc_confirmed в grouping - ["sub_id_12", "day"])
        params_all: dict() - параметры запроса в keitro_parser в external для считывания данных по общему потоку (в metrics обязана быть epc_confirmed в grouping - ["sub_id_12", "day"])
    
    """
    from MVPanalytics.keitaro import keitaro_parser
    import matplotlib.pyplot as plt
    import numpy as np
    import pandas as pd
    
    pars = keitaro_parser(**params_chan).sort_values('day')
    
    
    pars_all = keitaro_parser(**params_all).sort_values('day')
    
    
    if type_ == 'FS':
        plt.plot(pars.loc[pars['sub_id_12']=='feed'].set_index('day')['epc_confirmed'], marker = 'o', label = 'feed')
        plt.plot(pars.loc[pars['sub_id_12']=='sms'].set_index('day')['epc_confirmed'], marker = 'o', label = 'sms')

        plt.legend()
        plt.title(external_name)
        plt.xlabel('date')
        plt.ylabel('EPC')
        plt.xticks(rotation = 45)

        plt.show()
    elif type_ == 'FF':
        plt.plot(pars_all.loc[pars_all['sub_id_12']=='feed'].set_index('day')['epc_confirmed'], marker = 'o', label = 'feed all')
        plt.plot(pars.loc[pars['sub_id_12']=='feed'].set_index('day')['epc_confirmed'], marker = 'o', label = 'feed '+external_name)


        plt.legend()
        plt.title(f'{external_name} vs all')
        plt.xlabel('date')
        plt.ylabel('EPC')
        plt.xticks(rotation = 45)

        plt.show()
    elif type_ == 'SS':
        plt.plot(pars_all.loc[pars_all['sub_id_12']=='sms'].set_index('day')['epc_confirmed'], marker = 'o', label = 'sms all')
        plt.plot(pars.loc[pars['sub_id_12']=='sms'].set_index('day')['epc_confirmed'], marker = 'o', label = 'sms '+external_name)


        plt.legend()
        plt.title(f'{external_name} vs all')
        plt.xlabel('date')
        plt.ylabel('EPC')
        plt.xticks(rotation = 45)

        plt.show()
    else:
        return(print('ОШИБКА: Неверный тип графика. Тип должен быть "FS", "FF", "SS"'))


def weightgiver(api_key, host, comp_id, str_id,  perc_min, per, ban=[], max_ban=0.5, prime = [], prime_add = 0.1): 
    """
        Функция для раздачи офферам в партнёрке весов (сплитование)
    
        Args:
            api_key: (str) - апи-ключ для доступа в кейтаро
            host: (str) - url для доступа к кейтаро
            comp_id: (int) - id кампании
            str_id: (int) - id потока
            perc_min: (float [0, 1]) - минимальная доля сплита рекламодателя от суммарного epc
            per: (int) - интервал часов з который парсится информация
            ban: (list) - список из id рекламодателей, которые не в приоритете
            max_ban: (float [0, 1]) - какая доля отстаётся у неприроритетных оферов в случае, если суммарно на все неприоритетные оффера отводится больше указанной доли.
            prime: (list) - список из id рекламодателей, которые в главном приоритете
            prime_add: (float) - какая доля отнимается у всех неприоритетных оферов (например, если prime_add = 0.1, то 10% отнимается у всех приоритетов и добавляются к prime)
    """
    
    import numpy as np
    from MVPanalytics.keitaro import keitaro_parser
    from datetime import datetime, timedelta
    t=0
    while (t == 0):
        spl = keitaro_parser(api_key, host,
                interval= 'custom_date_range',
                firstdate=  (datetime.now() - timedelta(hours=int(per))).strftime("%Y-%m-%d %H:%M"),
                lastdate= datetime.now().strftime("%Y-%m-%d %H:%M"),
                filters = [{'name': "stream_id", 'operator': "EQUALS", 'expression': int(str_id)},
                        {'name': "campaign_id", 'operator': "EQUALS", 'expression': int(comp_id)}],
            grouping = ["stream", "offer", 'offer_id','affiliate_network', 'affiliate_network_id'],
            metrics = ['epc_confirmed', 'clicks']
            )
        
        t = spl['epc_confirmed'].max()
        per+=12
    if len(spl)>0:    
        spl['perc'] = (spl.epc_confirmed*100/(spl.epc_confirmed.sum())).round()
        spl['epc_help'] = spl['epc_confirmed']
        
        if (spl.loc[spl['affiliate_network_id'].isin(ban)]['perc'].sum()>max_ban):
            add_value = spl.loc[spl['affiliate_network_id'].isin(ban),'epc_help']*(1-max_ban)
                
            spl.loc[spl['affiliate_network_id'].isin(ban), 'epc_help'] = spl.loc[spl['affiliate_network_id'].isin(ban), 'epc_help']*(max_ban)
            spl.loc[~(spl['affiliate_network_id'].isin(ban)), 'epc_help'] = spl.loc[~(spl['affiliate_network_id'].isin(ban)), 'epc_help']+add_value.sum()/len(spl.loc[~(spl['affiliate_network_id'].isin(ban)), 'epc_help'])
            
            spl['perc'] = (spl.epc_help*100/(spl.epc_help.sum())).round()
        
        if (spl['affiliate_network_id'].isin(prime).sum()!=0):
            add_value = spl.loc[~spl['affiliate_network_id'].isin(prime),'epc_help']*(prime_add)
                
            spl.loc[~(spl['affiliate_network_id'].isin(prime)), 'epc_help'] = spl.loc[~(spl['affiliate_network_id'].isin(prime)),'epc_help']*(1-prime_add)
            spl.loc[spl['affiliate_network_id'].isin(prime), 'epc_help'] = spl.loc[spl['affiliate_network_id'].isin(prime), 'epc_help']+add_value.sum()/len(spl.loc[spl['affiliate_network_id'].isin(prime), 'epc_help'])
            
            spl['perc'] = (spl.epc_help*100/(spl.epc_help.sum())).round()
        if sum(spl['perc']<perc_min)!=0:
            add_value = spl.loc[~(spl['perc']<perc_min),'epc_help']*(perc_min)
                
            spl.loc[~(spl['perc']<perc_min), 'epc_help'] = spl.loc[~(spl['perc']<perc_min),'epc_help']*(1-perc_min)
            spl.loc[spl['perc']<perc_min, 'epc_help'] = spl.loc[spl['perc']<perc_min, 'epc_help']+add_value.sum()/len(spl.loc[spl['perc']<perc_min, 'epc_help'])
            
            spl['perc'] = (spl.epc_help*100/(spl.epc_help.sum())).round()
        if spl['perc'].sum()!=100:
            spl.loc[spl['perc'].idxmax(),'perc'] = spl['perc'].max() + (100-spl['perc'].sum())
    return(spl)

def off_analyzer(data,target, report_type, showcases = False):
    
    """
        Функция, позволяющая выявлять витрины по потоку (или офферу), на которых его: target < EPC, 0 < EPC < target, EPC=0
        
        Args:
            data: (pd.Dataframe) - спаршенные данные с кейтаро, которые должны содержать колонки ['epc_confirmed', 'external_id', 'clicks', 'sale_revenue']
            target: (float) - нижнее удовлетворяющее значеие EPC
            showcases: (bool) - нужна ли подробная таблица, которая выдаёт сами external с epc ниже таргета (рботает для report_type == 'offer')
            report_type: (str) - Вид отчёта, если 'city' - делает разбивку по городам, если 'offer' - то отчёт по !Одному! офферу. Отчёт по городам рабоатет для всех офферов
            
            Выдача:
                Если showcases = False/report_type == 'city' : Возвращает одну таблицу, которая показывает сколько кликов отливается на витрины с epc по потоку/потоку+городу target < EPC, 0 < EPC < target, EPC=0 без уточнения того, какие это витрины
                Если showcases = True: Возвращает 2 таблицы, первая - как из showcases = False, вторая - вместе с external_id
    """
    
    import pandas as pd
    import numpy as np
    if report_type == 'offer':
        data['more'] = data['epc_confirmed'].apply(lambda x: f'more than {target} epc' if x >target else( f'less then {target}' if ((x<target)&(x>0))  else 'zero_epc'))
        gr = data[['more', 'epc_confirmed','clicks','sale_revenue']].groupby(by='more').agg({'clicks':['sum','mean','max'], 'sale_revenue':'sum', 'epc_confirmed':['min', 'max']})

        gr['epc_agregated'] = gr[('sale_revenue','sum')]/gr[('clicks', 'sum')]
        if showcases == False:
            return(gr)
        else:
            gr_1 = data[['more', 'external_id','clicks','epc_confirmed']].groupby(by=['more','external_id']).sum().sort_values(by=['more','clicks'], ascending=False)
            return(gr, gr_1)
    
    if report_type == 'city':
        gr = data[['offer','city','epc_confirmed','clicks','sale_revenue']].groupby(by=['offer','city']).agg({'clicks':['sum','mean','max'], 'sale_revenue':'sum', 'epc_confirmed':['min', 'max']}).sort_values(by = ['offer',('clicks', 'sum')],ascending = False)

        gr['epc_agregated'] = gr[('sale_revenue','sum')]/gr[('clicks', 'sum')]
    
        gr[('clicks', '% of offer')] = 100*(data[['offer', 'city','clicks']].groupby(['offer', 'city']).sum())/(data[['offer', 'clicks']].groupby('offer').sum())
        return(gr.round(2))

def streamdiff(params_pars, target, metric, channel,minclick):
    '''
    Скрипт позволяет смотреть, сколько в рамках какого-то канала(витрина, вебмастер, пртнёр и тд) за промежуток времени(указыывается в параметрах запроса) было моментов в ремени в который целевая метрика была больше и меньше таргета
    
    Args:
    params_pars: dict() - параметры запроса функции keitaro_parser
    target: float - целевое значение с которым сравнивается
    metrics: string - метрика, по которой будет подсчёт
    channel: string - то, по чему должна идти группировка (stream/offer...)
    minclick: int - минимальное количество кликов, при который наблюдение будет взято для подсчёта сводной таблицы (напимер, если minclick=10, то в выборку не пойдут дни(если группировка по дням) в которые на потоке (если channel = 'streaam') было строго больше эктого количествоа кликов)
    
    '''
    from MVPanalytics.keitaro import keitaro_parser
    import matplotlib.pyplot as plt 
    import numpy as np 
    import pandas as pd 
    strm = keitaro_parser(**params_pars)
    strm = strm.loc[strm['clicks']>minclick]
    retr = strm[strm[metric] > target].groupby(channel).count()[[metric]].merge(strm[strm[metric] < target].groupby(channel).count()[[metric]], right_index=True, left_index=True, how = 'outer', suffixes = ['_more '+str(target), '_less '+str(target)]).fillna(0)
    retr['percent >'+str(target)] = retr[metric+'_more '+str(target)]*100/(retr[metric+'_more '+str(target)] + retr[metric+'_less '+str(target)])
    retr = retr.merge(strm[[channel, 'clicks']].groupby(channel).sum()['clicks'], right_index = True, left_index = True)
    if metric == 'epc_confirmed':
        retr = retr.merge(strm[[channel, 'sale_revenue']].groupby(channel).sum()['sale_revenue'], right_index = True, left_index = True)
        retr[metric+' for period'] = retr['sale_revenue']/retr['clicks']
        retr.drop(columns=['sale_revenue'], inplace=True)
    return(retr.sort_values('percent >'+str(target), ascending = 0))